@@ -41,17 +41,17 @@ contains
     type(hecmwST_matrix)                 :: hecMAT !< type hecmwST_matrix
     type(fstr_solid)                     :: fstrSOLID !< type fstr_solid
     type(hecmwST_matrix_lagrange)        :: hecLagMAT !< type hecmwST_matrix_lagrange
-    integer(kind=kint)                   :: ctsurf, etype, nnode, ndLocal(9) !< contents of type tContact
+    integer(kind=kint)                   :: ctsurf, etype, nnode, ndLocal(21) !< contents of type tContact
     integer(kind=kint)                   :: i, j, k, nlag, id_lagrange, grpid, algtype
     real(kind=kreal)                     :: lagrange
-    real(kind=kreal)                     :: stiffness(9*3 + 1, 9*3 + 1)
+    real(kind=kreal)                     :: stiffness(21*3 + 1, 21*3 + 1)
 
     hecLagMAT%AL_lagrange = 0.0d0
     hecLagMAT%AU_lagrange = 0.0d0
 
     id_lagrange = 0
 
-    do i = 1, size(fstrSOLID%contacts)
+    do i = 1, fstrSOLID%n_contacts
 
       grpid = fstrSOLID%contacts(i)%group
       if( .not. fstr_isContactActive( fstrSOLID, grpid, cstep ) ) cycle
@@ -87,6 +87,33 @@ contains
       enddo
     enddo
 
+    do i = 1, fstrSOLID%n_embeds
+
+      grpid = fstrSOLID%embeds(i)%group
+      if( .not. fstr_isEmbedActive( fstrSOLID, grpid, cstep ) ) cycle
+
+      do j = 1, size(fstrSOLID%embeds(i)%slave)
+
+        if( fstrSOLID%embeds(i)%states(j)%state == CONTACTFREE ) cycle
+
+        ctsurf = fstrSOLID%embeds(i)%states(j)%surface
+        etype = fstrSOLID%embeds(i)%master(ctsurf)%etype
+        nnode = size(fstrSOLID%embeds(i)%master(ctsurf)%nodes)
+        ndLocal(1) = fstrSOLID%embeds(i)%slave(j)
+        ndLocal(2:nnode+1) = fstrSOLID%embeds(i)%master(ctsurf)%nodes(1:nnode)
+
+        do k=1,3
+          id_lagrange = id_lagrange + 1
+          lagrange = hecLagMAT%Lagrange(id_lagrange)
+  
+          call getTiedStiffness(etype,nnode,k,fstrSOLID%embeds(i)%states(j),lagrange,stiffness)
+    
+          ! Assemble contact stiffness matrix of contact pair into global stiffness matrix
+          call hecmw_mat_ass_contactlag(nnode,ndLocal,id_lagrange,0.d0,stiffness,hecMAT,hecLagMAT)
+        enddo
+      enddo
+    enddo
+
   end subroutine fstr_AddContactStiffness
 
   !> \brief This subroutine obtains contact stiffness matrix of contact pair
@@ -96,7 +123,7 @@ contains
     integer(kind=kint)  :: idof
     type(tContactState) :: ctState !< type tContactState
     real(kind=kreal)    :: lagrange !< value of Lagrange multiplier
-    real(kind=kreal)    :: stiffness(9*3 + 1, 9*3 + 1) !< contact stiffness matrix
+    real(kind=kreal)    :: stiffness(:,:) !< contact stiffness matrix
 
     integer(kind=kint)  :: i, j, k, l
     real(kind=kreal)    :: shapefunc(nnode) !< normal vector at target point; shape functions
@@ -132,7 +159,7 @@ contains
     real(kind=kreal)    :: lagrange !< value of Lagrange multiplier
     real(kind=kreal)    :: tf_trial(3), length_tft
     real(kind=kreal)    :: tangent(3), tTm((nnode + 1)*3)
-    real(kind=kreal)    :: stiffness(9*3 + 1, 9*3 + 1) !< contact stiffness matrix
+    real(kind=kreal)    :: stiffness(:,:) !< contact stiffness matrix
 
     stiffness = 0.0d0
 
@@ -233,13 +260,13 @@ contains
     type(fstr_solid)                     :: fstrSOLID !< type fstr_solid
     type(hecmwST_matrix_lagrange) :: hecLagMAT !< type hecmwST_matrix_lagrange
     type(hecmwST_matrix)                 :: conMAT !< type hecmwST_matrix for contact part only
-    integer(kind=kint) :: ctsurf, etype, nnode, ndLocal(9) !< contents of type tContact
+    integer(kind=kint) :: ctsurf, etype, nnode, ndLocal(21) !< contents of type tContact
     integer(kind=kint) :: i, j, k, nlag, id_lagrange, algtype
-    real(kind=kreal)   :: ndCoord(9*3) !< nodal coordinates
-    real(kind=kreal)   :: ndu(9*3), ndDu(9*3) !< nodal displacement and its increment
+    real(kind=kreal)   :: ndCoord(21*3) !< nodal coordinates
+    real(kind=kreal)   :: ndu(21*3), ndDu(21*3) !< nodal displacement and its increment
     real(kind=kreal)   :: lagrange !< value of Lagrange multiplier
-    real(kind=kreal)   :: ctNForce(9*3+1)     !< nodal normal contact force vector
-    real(kind=kreal)   :: ctTForce(9*3+1)     !< nodal tangential contact force vector
+    real(kind=kreal)   :: ctNForce(21*3+1)     !< nodal normal contact force vector
+    real(kind=kreal)   :: ctTForce(21*3+1)     !< nodal tangential contact force vector
 
     integer(kind=kint) :: cstep !< current calculation step
     integer(kind=kint) :: grpid
@@ -247,8 +274,9 @@ contains
     id_lagrange = 0
     if( associated(fstrSOLID%CONT_NFORCE) ) fstrSOLID%CONT_NFORCE(:) = 0.d0
     if( associated(fstrSOLID%CONT_FRIC) ) fstrSOLID%CONT_FRIC(:) = 0.d0
+    if( associated(fstrSOLID%EMBED_NFORCE) ) fstrSOLID%EMBED_NFORCE(:) = 0.d0
 
-    do i = 1, size(fstrSOLID%contacts)
+    do i = 1, fstrSOLID%n_contacts
 
       grpid = fstrSOLID%contacts(i)%group
       if( .not. fstr_isContactActive( fstrSOLID, grpid, cstep ) ) cycle
@@ -282,11 +310,12 @@ contains
             call getContactNodalForce(etype,nnode,ndCoord,ndDu,fstrSOLID%contacts(i)%states(j),    &
             fstrSOLID%contacts(i)%tPenalty,fstrSOLID%contacts(i)%fcoeff,lagrange,ctNForce,ctTForce,.true.)
             ! Update non-eqilibrited force vector
-            call update_NDForce_contact(nnode,ndLocal,id_lagrange,lagrange,ctNForce,ctTForce,fstrSOLID,conMAT)
+            call update_NDForce_contact(nnode,ndLocal,id_lagrange,lagrange,ctNForce,ctTForce,  &
+              &  conMAT,fstrSOLID%CONT_NFORCE,fstrSOLID%CONT_FRIC)
           else if( algtype == CONTACTTIED ) then
             call getTiedNodalForce(etype,nnode,k,ndu,fstrSOLID%contacts(i)%states(j),lagrange,ctNForce,ctTForce)
             ! Update non-eqilibrited force vector
-            call update_NDForce_contact(nnode,ndLocal,id_lagrange,-1.d0,ctNForce,ctTForce,fstrSOLID,conMAT)
+            call update_NDForce_contact(nnode,ndLocal,id_lagrange,-1.d0,ctNForce,ctTForce,conMAT,fstrSOLID%CONT_NFORCE)
           endif 
 
         enddo
@@ -294,6 +323,40 @@ contains
 
     enddo
 
+    do i = 1, fstrSOLID%n_embeds
+
+      grpid = fstrSOLID%embeds(i)%group
+      if( .not. fstr_isEmbedActive( fstrSOLID, grpid, cstep ) ) cycle
+
+      do j = 1, size(fstrSOLID%embeds(i)%slave)
+
+        if( fstrSOLID%embeds(i)%states(j)%state == CONTACTFREE ) cycle
+
+        ctsurf = fstrSOLID%embeds(i)%states(j)%surface
+        etype = fstrSOLID%embeds(i)%master(ctsurf)%etype
+        nnode = size(fstrSOLID%embeds(i)%master(ctsurf)%nodes)
+        ndLocal(1) = fstrSOLID%embeds(i)%slave(j)
+        ndLocal(2:nnode+1) = fstrSOLID%embeds(i)%master(ctsurf)%nodes(1:nnode)
+        do k = 1, nnode+1
+          ndDu((k-1)*3+1:(k-1)*3+3) = fstrSOLID%dunode((ndLocal(k)-1)*3+1:(ndLocal(k)-1)*3+3)
+          ndu((k-1)*3+1:(k-1)*3+3) = fstrSOLID%unode((ndLocal(k)-1)*3+1:(ndLocal(k)-1)*3+3) + ndDu((k-1)*3+1:(k-1)*3+3)
+          ndCoord((k-1)*3+1:(k-1)*3+3) = hecMESH%node((ndLocal(k)-1)*3+1:(ndLocal(k)-1)*3+3) + ndu((k-1)*3+1:(k-1)*3+3)
+        enddo
+
+        do k=1,3
+          id_lagrange = id_lagrange + 1
+          lagrange = hecLagMAT%Lagrange(id_lagrange)
+  
+          fstrSOLID%embeds(i)%states(j)%multiplier(k) = hecLagMAT%Lagrange(id_lagrange)
+    
+          call getTiedNodalForce(etype,nnode,k,ndu,fstrSOLID%embeds(i)%states(j),lagrange,ctNForce,ctTForce)
+          ! Update non-eqilibrited force vector
+          call update_NDForce_contact(nnode,ndLocal,id_lagrange,-1.d0,ctNForce,ctTForce,conMAT,fstrSOLID%EMBED_NFORCE)
+
+        enddo
+      enddo
+    enddo
+
     !    Consider SPC condition
     call fstr_Update_NDForce_SPC(cstep, hecMESH, fstrSOLID, hecMAT%B)
     call fstr_Update_NDForce_SPC(cstep, hecMESH, fstrSOLID, conMAT%B)
@@ -318,7 +381,7 @@ contains
     ctNForce = 0.0d0
     ctTForce = 0.0d0
 
-    call getShapeFunc( etype, ctState%lpos(:), shapefunc )
+    call getShapeFunc( etype, ctState%lpos, shapefunc )
 
     nTm = 0.d0
     nTm(idof) = -1.d0
@@ -352,7 +415,7 @@ contains
     ctNForce = 0.0d0
     ctTForce = 0.0d0
 
-    call getShapeFunc( etype, ctState%lpos(:), shapefunc )
+    call getShapeFunc( etype, ctState%lpos, shapefunc )
 
     normal(1:3) = ctState%direction(1:3)
 
@@ -436,9 +499,8 @@ contains
 
   !> \brief This subroutine assembles contact nodal force vector into right-hand side vector
   !! to update non-equilibrated nodal force vector.
-  subroutine update_NDForce_contact(nnode,ndLocal,id_lagrange,lagrange,ctNForce,ctTForce,fstrSOLID,hecMAT)
+  subroutine update_NDForce_contact(nnode,ndLocal,id_lagrange,lagrange,ctNForce,ctTForce,hecMAT,cont_nforce,cont_fric)
 
-    type(fstr_solid)                        :: fstrSOLID                       !< type fstr_solid
     type(hecmwST_matrix)                 :: hecMAT !< type hecmwST_matrix
     integer(kind=kint) :: nnode, ndLocal(nnode + 1) !< number of nodes of master segment
     !< global number of nodes of contact pair
@@ -448,6 +510,8 @@ contains
     integer (kind=kint)                     :: i, inod, idx
     real(kind=kreal)                        :: ctNForce((nnode+1)*3+1)         !< contact force vector
     real(kind=kreal)                        :: ctTForce((nnode+1)*3+1)         !< contact force vector
+    real(kind=kreal), pointer               :: cont_nforce(:)         !< contact force vector
+    real(kind=kreal), pointer, optional     :: cont_fric(:)         !< contact force vector
 
     np = hecMAT%NP; ndof = hecMAT%NDOF
 
@@ -455,9 +519,8 @@ contains
       inod = ndLocal(i)
       idx = (inod-1)*3+1
       hecMAT%B(idx:idx+2) = hecMAT%B(idx:idx+2) + ctNForce((i-1)*3+1:(i-1)*3+3) + ctTForce((i-1)*3+1:(i-1)*3+3)
-      if( lagrange < 0.d0 ) cycle
-      fstrSOLID%CONT_NFORCE(idx:idx+2) = fstrSOLID%CONT_NFORCE(idx:idx+2) + ctNForce((i-1)*3+1:(i-1)*3+3)
-      fstrSOLID%CONT_FRIC(idx:idx+2) = fstrSOLID%CONT_FRIC(idx:idx+2) + ctTForce((i-1)*3+1:(i-1)*3+3)
+      cont_nforce(idx:idx+2) = cont_nforce(idx:idx+2) + ctNForce((i-1)*3+1:(i-1)*3+3)
+      if( present(cont_fric) ) cont_fric(idx:idx+2) = cont_fric(idx:idx+2) + ctTForce((i-1)*3+1:(i-1)*3+3)
     enddo
 
     hecMAT%B(np*ndof+id_lagrange) = ctNForce((nnode+1)*3+1)+ctTForce((nnode+1)*3+1)
@@ -476,22 +539,22 @@ contains
     integer(kind=kint) :: cstep !< current step
     integer(kind=kint) :: np, ndof !< total number of nodes; degree of freedom
     integer(kind=kint) :: i, j, k, l, id_lagrange, lnod, grpid
-    integer(kind=kint) :: ctsurf, etype, nnode, ndLocal(9) !< contents of type tContact
-    real(kind=kreal)   :: ndu(9*3), ndCoord(9*3), lagrange !< nodal coordinates; value of Lagrange mutiplier
-    real(kind=kreal)   :: normal(3), shapefunc(9) !< normal vector; shape functions
+    integer(kind=kint) :: ctsurf, etype, nnode, ndLocal(21) !< contents of type tContact
+    real(kind=kreal)   :: ndu(21*3), ndCoord(21*3), lagrange !< nodal coordinates; value of Lagrange mutiplier
+    real(kind=kreal)   :: normal(3), shapefunc(21) !< normal vector; shape functions
     real(kind=kreal)   :: nTm(10*3) !< vector
     real(kind=kreal)   :: tf_final(3) !< final friciton force vector
-    real(kind=kreal)   :: ctForce(9*3 + 1) !< initial nodal contact force vector
+    real(kind=kreal)   :: ctForce(21*3 + 1) !< initial nodal contact force vector
 
     integer(kind=kint) :: algtype, nlag    
-    real(kind=kreal)   :: ctNForce(9*3+1)     !< nodal normal contact force vector
-    real(kind=kreal)   :: ctTForce(9*3+1)     !< nodal tangential contact force vector
+    real(kind=kreal)   :: ctNForce(21*3+1)     !< nodal normal contact force vector
+    real(kind=kreal)   :: ctTForce(21*3+1)     !< nodal tangential contact force vector
 
     np = hecMAT%NP ; ndof = hecMAT%NDOF
 
     id_lagrange = 0
 
-    do i = 1, size(fstrSOLID%contacts)
+    do i = 1, fstrSOLID%n_contacts
 
       grpid = fstrSOLID%contacts(i)%group
       if( .not. fstr_isContactActive( fstrSOLID, grpid, cstep ) ) cycle
@@ -522,17 +585,50 @@ contains
             call getContactNodalForce(etype,nnode,ndCoord,ndu,fstrSOLID%contacts(i)%states(j),    &
             fstrSOLID%contacts(i)%tPenalty,fstrSOLID%contacts(i)%fcoeff,lagrange,ctNForce,ctTForce,.false.)
             ! Update non-eqilibrited force vector
-            call update_NDForce_contact(nnode,ndLocal,id_lagrange,lagrange,ctNForce,ctTForce,fstrSOLID,hecMAT)
+            call update_NDForce_contact(nnode,ndLocal,id_lagrange,lagrange,ctNForce,ctTForce, & 
+               &  hecMAT,fstrSOLID%CONT_NFORCE,fstrSOLID%CONT_FRIC)
           else if( algtype == CONTACTTIED ) then
             call getTiedNodalForce(etype,nnode,k,ndu,fstrSOLID%contacts(i)%states(j),lagrange,ctNForce,ctTForce)
             ! Update non-eqilibrited force vector
-            call update_NDForce_contact(nnode,ndLocal,id_lagrange,-1.d0,ctNForce,ctTForce,fstrSOLID,hecMAT)
+            call update_NDForce_contact(nnode,ndLocal,id_lagrange,-1.d0,ctNForce,ctTForce,hecMAT,fstrSOLID%CONT_NFORCE)
           endif 
 
         enddo
       enddo
     enddo
 
+    do i = 1, fstrSOLID%n_embeds
+
+      grpid = fstrSOLID%embeds(i)%group
+      if( .not. fstr_isEmbedActive( fstrSOLID, grpid, cstep ) ) cycle
+
+
+      do j = 1, size(fstrSOLID%embeds(i)%slave)
+
+        if( fstrSOLID%embeds(i)%states(j)%state == CONTACTFREE ) cycle
+
+        ctsurf = fstrSOLID%embeds(i)%states(j)%surface
+        etype = fstrSOLID%embeds(i)%master(ctsurf)%etype
+        nnode = size(fstrSOLID%embeds(i)%master(ctsurf)%nodes)
+        ndLocal(1) = fstrSOLID%embeds(i)%slave(j)
+        ndLocal(2:nnode+1) = fstrSOLID%embeds(i)%master(ctsurf)%nodes(1:nnode)
+        do k = 1, nnode+1
+          ndu((k-1)*3+1:(k-1)*3+3) = fstrSOLID%unode((ndLocal(k)-1)*3+1:(ndLocal(k)-1)*3+3)
+          ndCoord((k-1)*3+1:(k-1)*3+3) = hecMESH%node((ndLocal(k)-1)*3+1:(ndLocal(k)-1)*3+3) + ndu((k-1)*3+1:(k-1)*3+3)
+        enddo
+
+        do k=1,3
+          id_lagrange = id_lagrange + 1
+          lagrange = fstrSOLID%embeds(i)%states(j)%multiplier(k)
+    
+          call getTiedNodalForce(etype,nnode,k,ndu,fstrSOLID%embeds(i)%states(j),lagrange,ctNForce,ctTForce)
+          ! Update non-eqilibrited force vector
+          call update_NDForce_contact(nnode,ndLocal,id_lagrange,-1.d0,ctNForce,ctTForce,hecMAT,fstrSOLID%EMBED_NFORCE)
+
+        enddo
+      enddo
+    enddo
+
   end subroutine fstr_ass_load_contact
 
 end module m_addContactStiffness
