@@ -230,9 +230,6 @@ contains
       deallocate( func, inv_func )
     enddo !<element type loop
 
-    if( fstrSOLID%is_smoothing_active ) call fstr_NodalStress3D_C3D4_SESNS( &
-      &  hecMESH, fstrSOLID, nnumber, fstrSOLID%STRAIN, fstrSOLID%STRESS, fstrSOLID%ESTRAIN, fstrSOLID%ESTRESS )
-
     !C** calculate nodal stress and strain
     do i = 1, hecMESH%n_node
       if( nnumber(i) == 0 ) cycle
@@ -243,6 +240,9 @@ contains
       endif
     enddo
 
+    if( fstrSOLID%is_smoothing_active ) call fstr_NodalStress3D_C3D4_SESNS( &
+      &  hecMESH, fstrSOLID, nnumber, fstrSOLID%STRAIN, fstrSOLID%STRESS, fstrSOLID%ESTRAIN, fstrSOLID%ESTRESS )
+
     if( flag33 == 1 )then
       do nlyr = 1, ntot_lyr
         do i = 1, hecMESH%n_node
@@ -304,7 +304,7 @@ contains
   subroutine fstr_NodalStress3D_C3D4_SESNS( hecMESH, fstrSOLID, nnumber, &
       Nodal_STRAIN, Nodal_STRESS, Elemental_STRAIN, Elemental_STRESS )
     type(hecmwST_local_mesh),intent(in) :: hecMESH
-    type(fstr_solid),intent(in)         :: fstrSOLID
+    type(fstr_solid),intent(inout)         :: fstrSOLID
     integer(kind=kint), allocatable, intent(inout) :: nnumber(:)
     real(kind=kreal), pointer, intent(inout)   :: Nodal_STRAIN(:)
     real(kind=kreal), pointer, intent(inout)   :: Nodal_STRESS(:)
@@ -431,8 +431,13 @@ contains
       stress_hyd_ndave(1:6) = stress_hyd_ndave(1:6)/nsecdup
       stress_dev_ndave(1:6) = stress_dev_ndave(1:6)/nsecdup
 
-      Nodal_STRAIN(6*i-5:6*i) = Nodal_STRAIN(6*i-5:6*i)+strain_hyd_ndave(1:6)
-      Nodal_STRESS(6*i-5:6*i) = Nodal_STRESS(6*i-5:6*i)+stress_hyd_ndave(1:6)+stress_dev_ndave(1:6)
+      if( nnumber(i) == 0 ) then
+        Nodal_STRAIN(6*i-5:6*i) = strain_hyd_ndave(1:6)
+        Nodal_STRESS(6*i-5:6*i) = stress_hyd_ndave(1:6)+stress_dev_ndave(1:6)
+      else
+        Nodal_STRAIN(6*i-5:6*i) = 0.5d0*(Nodal_STRAIN(6*i-5:6*i)+strain_hyd_ndave(1:6))
+        Nodal_STRESS(6*i-5:6*i) = 0.5d0*(Nodal_STRESS(6*i-5:6*i)+stress_hyd_ndave(1:6)+stress_dev_ndave(1:6))
+      endif
     end do
 
     ! ELEMENTAL STRAIN and STRESS
