@@ -71,7 +71,7 @@ contains
     if( associated( fstrSOLID%contacts ) ) then
       call hecmw_restart_read_int(nif)
       contactNode = nif(1)
-      do i= 1, size(fstrSOLID%contacts)
+      do i= 1, fstrSOLID%n_contacts
         do j= 1, size(fstrSOLID%contacts(i)%slave)
           call hecmw_restart_read_int(nif)
           fstrSOLID%contacts(i)%states(j)%surface = nif(1)
@@ -201,7 +201,7 @@ contains
     if( associated( fstrSOLID%contacts ) ) then
       nif(1) = contactNode
       call hecmw_restart_add_int(nif,size(nif))
-      do i= 1, size(fstrSOLID%contacts)
+      do i= 1, fstrSOLID%n_contacts
         do j= 1, size(fstrSOLID%contacts(i)%slave)
           nif(1) = fstrSOLID%contacts(i)%states(j)%surface
           nif(2) = fstrSOLID%contacts(i)%states(j)%state
@@ -263,7 +263,7 @@ contains
     if(present(contactNode)) then
       call hecmw_restart_read_int(nif)
       contactNode = nif(1)
-      do i= 1, size(fstrSOLID%contacts)
+      do i= 1, fstrSOLID%n_contacts
         do j= 1, size(fstrSOLID%contacts(i)%slave)
           call hecmw_restart_read_int(nif)
           fstrSOLID%contacts(i)%states(j)%surface = nif(1)
@@ -349,7 +349,7 @@ contains
     if(present(contactNode)) then
       nif(1) = contactNode
       call hecmw_restart_add_int(nif,size(nif))
-      do i= 1, size(fstrSOLID%contacts)
+      do i= 1, fstrSOLID%n_contacts
         do j= 1, size(fstrSOLID%contacts(i)%slave)
           nif(1) = fstrSOLID%contacts(i)%states(j)%surface
           nif(2) = fstrSOLID%contacts(i)%states(j)%state
