@@ -277,9 +277,9 @@ contains
     if( ISUF==1 ) then
       ! INTEGRATION OVER SURFACE
       do I=1,NSUR
-        elecoord(1,i)=XX(NOD(I))
-        elecoord(2,i)=YY(NOD(i))
-        elecoord(3,i)=ZZ(NOD(i))
+        elecoord(1,I)=XX(NOD(I))
+        elecoord(2,I)=YY(NOD(I))
+        elecoord(3,I)=ZZ(NOD(I))
       enddo
       do IG2=1,NumOfQuadPoints( SURTYPE )
         call getQuadPoint( SURTYPE, IG2, localcoord(1:2) )
@@ -330,9 +330,9 @@ contains
           PHX=XCOD-HX
           PHY=YCOD-HY
           PHZ=ZCOD-HZ
-          COEFX=RHO*val*val*PHX
-          COEFY=RHO*val*val*PHY
-          COEFZ=RHO*val*val*PHZ
+          COEFX=RHO*val*dabs(val)*PHX
+          COEFY=RHO*val*dabs(val)*PHY
+          COEFZ=RHO*val*dabs(val)*PHZ
         end if
 
         WG=getWeight( etype, LX )*DET
@@ -620,7 +620,7 @@ contains
 
     if( flag == INFINITESIMAL ) then
 
-      gauss%stress(1:6) = matmul( D(1:6, 1:6), dstrain(1:6) )
+      gauss%stress(1:6) = gauss%stress_bak(1:6) + matmul( D(1:6, 1:6), dstrain(1:6)-gauss%strain_bak(1:6) )
       if( isViscoelastic(mtype) .AND. tincr /= 0.0D0 ) then
         call StressUpdate( gauss, D3, dstrain, gauss%stress, coordsys, time, tincr, ttc, ttn, hdflag=hdflag_in )
         gauss%stress = real(gauss%stress)
@@ -654,7 +654,7 @@ contains
 
         !stress integration
         trD = dstrain(1)+dstrain(2)+dstrain(3)
-        dum(:,:) = dumstress + matmul( rot,dumstress ) - matmul( dumstress, rot ) + dumstress*trD
+        dum(:,:) = dumstress + matmul( rot,dumstress ) - matmul( dumstress, rot ) - dumstress*trD
         !call Hughes_Winget_rotation_3D( rot, dumstress, dum )
 
         gauss%stress(1) = dum(1,1) + dstress(1)
