@@ -4,30 +4,46 @@
  *****************************************************************************/
 
 /**
- * @brief 並列計算された結果を読込み処理するためのユーティリティ
+ * @brief Utility for reading and processing results computed in parallel
  */
 
 #include "fstr_rmerge_util.h"
 
-extern FILE* log_fp;
-extern int nrank;
-extern int strid;
-extern int endid;
-extern int intid;
+#include <stdio.h>
+#include <string.h>
+#include <stdlib.h>
 
-static void out_log(const char* fmt, ...) {
+#include "hecmw_util.h"
+#include "hecmw_dist_free.h"
+#include "hecmw_io_dist.h"
+#include "hecmw_io_get_mesh.h"
+#include "hecmw_etype.h"
+
+static FILE* Log_FP;
+
+/**
+ * @brief Set file pointer for log output
+ */
+void fstr_set_log_fp(FILE *log_fp) {
+  Log_FP = log_fp;
+}
+
+/**
+ * @brief Log output
+ */
+void fstr_out_log(const char* fmt, ...) {
   va_list arg;
   va_start(arg, fmt);
-  vfprintf(log_fp, fmt, arg);
+  vfprintf(Log_FP, fmt, arg);
   va_end(arg);
 }
 
 /**
- * @brief 全分散メッシュの読込み
+ * @brief Read all distributed meshes
  */
 
 static int get_dist_fname(char* name_ID, char* fheader, int* fg_single,
-                          int* refine, int irank) {
+                          int* refine, int nrank, int irank) {
   struct hecmw_ctrl_meshfiles* files;
 
   files = HECMW_ctrl_get_meshfiles_header_sub(name_ID, nrank, irank);
@@ -41,7 +57,7 @@ static int get_dist_fname(char* name_ID, char* fheader, int* fg_single,
       *fg_single = 1;
     }
     *refine = files->meshfiles[0].refine;
-    out_log("refine number is %d\n", *refine);
+    fstr_out_log("refine number is %d\n", *refine);
   } else {
     HECMW_ctrl_free_meshfiles(files);
     return -1;
@@ -58,14 +74,14 @@ static int get_area_n(char* fheader) {
 
   while (1) {
     sprintf(buff, "%s.%d", fheader, area);
-    out_log("try open : %s  ... ", buff);
+    fstr_out_log("try open : %s  ... ", buff);
     fp = fopen(buff, "r");
     if (!fp) {
-      out_log("fail\n");
-      out_log("area number is %d\n", area);
+      fstr_out_log("fail\n");
+      fstr_out_log("area number is %d\n", area);
       return area;
     } else {
-      out_log("success\n");
+      fstr_out_log("success\n");
       fclose(fp);
     }
     area++;
@@ -73,10 +89,11 @@ static int get_area_n(char* fheader) {
 }
 
 /**
- * @brief 全分散メッシュの読込み
+ * @brief Read all distributed meshes
  */
 
 struct hecmwST_local_mesh** fstr_get_all_local_mesh(char* name_ID,
+                                                    int nrank,
                                                     int* area_number,
                                                     int* refine) {
   int i;
@@ -86,17 +103,17 @@ struct hecmwST_local_mesh** fstr_get_all_local_mesh(char* name_ID,
   char fname[HECMW_FILENAME_LEN + 1];
   struct hecmwST_local_mesh** mesh;
 
-  if (get_dist_fname(name_ID, fheader, &fg_single, refine, 0)) return NULL;
+  if (get_dist_fname(name_ID, fheader, &fg_single, refine, nrank, 0)) return NULL;
 
   if (fg_single) {
-    out_log("mesh file type is NOT HECMW_DIST.\n");
+    fstr_out_log("mesh file type is NOT HECMW_DIST.\n");
     area_n = 1;
-    out_log("area number is %d\n", area_n);
+    fstr_out_log("area number is %d\n", area_n);
     mesh    = HECMW_malloc(area_n * sizeof(struct hecmwST_local_mesh*));
     mesh[0] = HECMW_get_mesh(name_ID);
     if (!mesh[0]) return NULL;
   } else {
-    out_log("mesh file type is HECMW_DIST.\n");
+    fstr_out_log("mesh file type is HECMW_DIST.\n");
     if (nrank == 0) {
       area_n = get_area_n(fheader);
     } else {
@@ -108,10 +125,10 @@ struct hecmwST_local_mesh** fstr_get_all_local_mesh(char* name_ID,
       if (nrank == 0) {
         sprintf(fname, "%s.%d", fheader, i);
       } else {
-        get_dist_fname(name_ID, fheader, &fg_single, refine, i);
+        get_dist_fname(name_ID, fheader, &fg_single, refine, nrank, i);
         sprintf(fname, "%s.%d", fheader, i);
       }
-      out_log("loading dist mesh from %s\n", fname);
+      fstr_out_log("loading dist mesh from %s\n", fname);
       mesh[i] = HECMW_get_dist_mesh(fname);
       if (!mesh[i]) return NULL;
     }
@@ -122,7 +139,7 @@ struct hecmwST_local_mesh** fstr_get_all_local_mesh(char* name_ID,
 }
 
 /**
- * @brief メッシュの削除
+ * @brief Delete mesh
  */
 
 void fstr_free_mesh(struct hecmwST_local_mesh** mesh, int area_n) {
@@ -138,17 +155,15 @@ void fstr_free_mesh(struct hecmwST_local_mesh** mesh, int area_n) {
 }
 
 /**
- * @brief ステップ数を調べる(ファイルの存在を調べる)
+ * @brief Check the number of steps (check for the existence of files)
  */
 
-int fstr_get_step_n(char* name_ID) {
+int fstr_get_step_n(char* name_ID, int nrank) {
   FILE* fp;
   int step, fg_text;
   char* fheader;
   char fname[HECMW_FILENAME_LEN + 1];
 
-  if (endid > -1) return endid;
-
   if (nrank == 0) {
     if ((fheader = HECMW_ctrl_get_result_fileheader(name_ID, 1, &fg_text)) ==
         NULL)
@@ -162,14 +177,14 @@ int fstr_get_step_n(char* name_ID) {
   step = 1;
   while (1) {
     sprintf(fname, "%s.0.%d", fheader, step);
-    out_log("try open : %s  ... ", fname);
+    fstr_out_log("try open : %s  ... ", fname);
     fp = fopen(fname, "r");
     if (!fp) {
-      out_log("fail\n");
-      out_log("step number is %d\n", step - 1);
+      fstr_out_log("fail\n");
+      fstr_out_log("step number is %d\n", step - 1);
       return step - 1;
     } else {
-      out_log("success\n");
+      fstr_out_log("success\n");
       fclose(fp);
     }
     step++;
@@ -177,11 +192,11 @@ int fstr_get_step_n(char* name_ID) {
 }
 
 /**
- * @brief ステップの全領域データの読み込み
+ * @brief Read all area data of step
  */
 
 fstr_res_info** fstr_get_all_result(char* name_ID, int step, int area_n,
-                                    int refine) {
+                                    int refine, int nrank) {
   char* fheader;
   char fname[HECMW_FILENAME_LEN + 1];
   fstr_res_info** res;
@@ -257,7 +272,7 @@ fstr_res_info** fstr_get_all_result(char* name_ID, int step, int area_n,
         if (node_gid[j] < 0 && node_gid[j] > node_gid[j - 1]) flag++;
       }
       count--;
-      out_log("\narea:%d -- refined_nn_internal:%d", i, count);
+      fstr_out_log("\narea:%d -- refined_nn_internal:%d", i, count);
       refine_nnode += count;
 
       count = 0;
@@ -293,7 +308,7 @@ fstr_res_info** fstr_get_all_result(char* name_ID, int step, int area_n,
       count = 0;
       for (j = 0; j < nelem; j++)
         if (elem_gid[j] > 0) count++;
-      out_log("\narea:%d -- ne_original from result:%d", i, count);
+      fstr_out_log("\narea:%d -- ne_original from result:%d", i, count);
       res[i]->nelem_gid = count;
       res[i]->result->elem_val_item =
           HECMW_malloc(num * count * sizeof(double));
@@ -336,13 +351,13 @@ fstr_res_info** fstr_get_all_result(char* name_ID, int step, int area_n,
     }
   }
 
-  if (refine) out_log("\ntotal refined_nn_internal:%d\n", refine_nnode);
+  if (refine) fstr_out_log("\ntotal refined_nn_internal:%d\n", refine_nnode);
 
   return res;
 }
 
 /**
- * @brief ステップの全領域データの結合
+ * @brief Combine data in all areas of the step
  */
 
 struct hecmwST_result_data* fstr_all_result(fstr_glt* glt, fstr_res_info** res,
@@ -423,7 +438,7 @@ struct hecmwST_result_data* fstr_all_result(fstr_glt* glt, fstr_res_info** res,
 }
 
 /**
- * @brief fstr_res_info の削除
+ * @biref Delete fstr_res_info
  */
 
 void fstr_free_result(fstr_res_info** res, int area_n) {
@@ -442,7 +457,7 @@ void fstr_free_result(fstr_res_info** res, int area_n) {
 }
 
 /**
- * @brief グローバルとローカル、所属領域のテーブル fstr_glt の作成
+ * @brief Create table for global ID, local ID and belonging area records fstr_glt
  */
 
 static int cmp_global_glt(const fstr_gl_rec* g1, const fstr_gl_rec* g2) {
@@ -462,10 +477,10 @@ fstr_glt* fstr_create_glt(struct hecmwST_local_mesh** mesh, int area_n) {
 
   all_n = 0;
   for (i = 0; i < area_n; i++) {
-    out_log("area:%d -- nn_internal:%d\n", i, mesh[i]->nn_internal);
+    fstr_out_log("area:%d -- nn_internal:%d\n", i, mesh[i]->nn_internal);
     all_n += mesh[i]->nn_internal;
   }
-  out_log("total nn_internal:%d\n", all_n);
+  fstr_out_log("total nn_internal:%d\n", all_n);
 
   nrec = HECMW_malloc(sizeof(fstr_gl_rec) * all_n);
   if (!nrec) return NULL;
@@ -495,14 +510,15 @@ fstr_glt* fstr_create_glt(struct hecmwST_local_mesh** mesh, int area_n) {
     for (j = 0; j < mesh[i]->ne_internal; j++) {
       eid = mesh[i]->elem_internal_list[j] - 1;
       if ( HECMW_is_etype_patch(area_etype_list[eid]) ) continue;
+      if ( HECMW_is_etype_link(area_etype_list[eid]) ) continue;
       area_e++;
     }
 
     HECMW_free(area_etype_list);
     all_e += area_e;
-    out_log("area:%d -- ne_internal:%d\n", i, area_e);
+    fstr_out_log("area:%d -- ne_internal:%d\n", i, area_e);
   }
-  out_log("total ne_internal:%d\n", all_e);
+  fstr_out_log("total ne_internal:%d\n", all_e);
 
   erec = HECMW_malloc(sizeof(fstr_gl_rec) * all_e);
   if (!erec) return NULL;
@@ -544,7 +560,7 @@ fstr_glt* fstr_create_glt(struct hecmwST_local_mesh** mesh, int area_n) {
 }
 
 /**
- * @brief fstr_glt の削除
+ * @brief Delete fstr_glt
  */
 
 void fstr_free_glt(fstr_glt* glt) {
@@ -557,7 +573,7 @@ void fstr_free_glt(fstr_glt* glt) {
 }
 
 /**
- * @brief 単一領域メッシュの作成
+ * @brief Create a single region mesh
  */
 
 struct hecmwST_local_mesh* fstr_create_glmesh(fstr_glt* glt) {
@@ -582,7 +598,7 @@ struct hecmwST_local_mesh* fstr_create_glmesh(fstr_glt* glt) {
 }
 
 /**
- * @brief 単一領域メッシュの削除
+ * @brief Delete a single region mesh
  */
 
 void fstr_free_glmesh(struct hecmwST_local_mesh* mesh) {
